@@ -947,7 +947,7 @@ chxj_calc_ecc(qr_code_t* qrcode,
   int exist_flag = 0;
   unsigned char** rs_block;
 #ifdef QR_CODE_DEBUG
-  ap_log_rerror(APLOG_MARK,APLOG_DEBUG, 0, r, "start chxj_calc_ecc()");
+  DBG(r,"start chxj_calc_ecc()");
 #endif
 
   rs_total_block_count = v_ecc_spec_table[qrcode->version*4+qrcode->level].rs[0].rs_block_count +
@@ -977,7 +977,7 @@ chxj_calc_ecc(qr_code_t* qrcode,
       ecc_count = v_ecc_spec_table[qrcode->version*4+qrcode->level].rs[rs_pos].total_code_count - data_count;
 
 #ifdef QR_CODE_DEBUG
-      ap_log_rerror(APLOG_MARK,APLOG_DEBUG, 0, r, "data_count[%d] ecc_count[%d]", data_count, ecc_count);
+      DBG2(r,"data_count[%d] ecc_count[%d]", data_count, ecc_count);
 #endif
 
       rs_block[now_rs_num] = (unsigned char*)apr_palloc(qrcode->r->pool, data_count + ecc_count + 1);
@@ -987,14 +987,14 @@ chxj_calc_ecc(qr_code_t* qrcode,
       rs_block_ecc_size[now_rs_num] = ecc_count;
 
 #ifdef QR_CODE_DEBUG
-      ap_log_rerror(APLOG_MARK,APLOG_DEBUG, 0, r, "apr_palloc() tmp");
+      DBG(r,"apr_palloc() tmp");
 #endif
 
       memset(tmp, 0, data_count + ecc_count + 1);
       memset(rs_block[now_rs_num], 0, data_count + ecc_count + 1);
 
 #ifdef QR_CODE_DEBUG
-      ap_log_rerror(APLOG_MARK,APLOG_DEBUG, 0, r, "memset end");
+      DBG(r,"memset end");
 #endif
 
       /* ǡ򥳥ԡ */
@@ -1013,7 +1013,7 @@ chxj_calc_ecc(qr_code_t* qrcode,
         for (jj=ecc_count + data_count - 1; jj>=0; jj--) {
           debug_rows = apr_pstrcat(r->pool, debug_rows, apr_psprintf(r->pool, "[%d]", tmp[jj]), NULL);
         }
-        ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "rows [%s]", debug_rows);
+        DBG1(r,"rows [%s]", debug_rows);
       } while(0);
       /* for DEBUG END */
 #endif
@@ -1024,11 +1024,10 @@ chxj_calc_ecc(qr_code_t* qrcode,
         int tgt = tmp[ii--];
         int shisu = v_galois_int_to_log[tgt];
 #ifdef QR_CODE_DEBUG
-        ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "tgt[%d] shisu[%d]", tgt, shisu);
+        DBG2(r,"tgt[%d] shisu[%d]", tgt, shisu);
 #endif
-        if (tgt == 0) {
+        if (tgt == 0)
           continue;
-        }
 
         for (jj=ecc_count; jj>=0;jj--) {
           assert(v_poly_ecc[ecc_count] != NULL);
@@ -1042,7 +1041,7 @@ chxj_calc_ecc(qr_code_t* qrcode,
           for (jj=ecc_count + data_count - 1; jj>=0; jj--) {
             debug_rows = apr_pstrcat(r->pool, debug_rows, apr_psprintf(r->pool, "[%d]", tmp[jj]), NULL);
           }
-          ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "rows [%s]", debug_rows);
+          DBG1(r,"rows [%s]", debug_rows);
         } while(0);
         /* for DEBUG END */
 #endif
@@ -1058,13 +1057,13 @@ chxj_calc_ecc(qr_code_t* qrcode,
 #ifdef QR_CODE_DEBUG
   /* for DEBUG */
   do {
-    ap_log_rerror(APLOG_MARK,APLOG_DEBUG, 0, r, "######### RS BLOCK DUMP ###############");
+    DBG(r,"######### RS BLOCK DUMP ###############");
     for (jj=0; jj<rs_total_block_count; jj++) {
        char* rows = apr_psprintf(r->pool, "%02d size:[%d] rest:[%d]",jj, rs_block_size[jj], rs_block_rest[jj]);
        for (ii=0; ii<rs_block_size[jj]; ii++) {
          rows = apr_pstrcat(r->pool, rows, apr_psprintf(r->pool, "[%d]", rs_block[jj][ii]), NULL);
        }
-       ap_log_rerror(APLOG_MARK,APLOG_DEBUG, 0, r, "%s", rows);
+       DBG(r,"%s", rows);
     }
   } while(0);
 #endif
@@ -1103,9 +1102,9 @@ chxj_calc_ecc(qr_code_t* qrcode,
 #ifdef QR_CODE_DEBUG
   /* for DEBUG */
   do {
-    ap_log_rerror(APLOG_MARK,APLOG_DEBUG, 0, r, "######### AFTER BLOCK DUMP ###############");
+    DBG(r,"######### AFTER BLOCK DUMP ###############");
     for (ii=0; ii<rslt_pos; ii++) {
-      ap_log_rerror(APLOG_MARK,APLOG_DEBUG, 0, r, "[%d]", dst[ii]);
+      DBG1(r,"[%d]", dst[ii]);
     }
   } while(0);
 #endif
