@@ -14,16 +14,31 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
+#if 0
+#include <wand/magick_wand.h>
+
+#undef PACKAGE_NAME
+#undef PACKAGE_STRING
+#undef PACKAGE_TARNAME
+#undef PACKAGE_VERSION
+
+#endif
+
 #include <libgen.h>
 #include "mod_chxj.h"
 #include "chxj_img_conv_format.h"
 #include "chxj_specified_device.h"
 #include "chxj_str_util.h"
 #include "chxj_qr_code.h"
+#include "chxj_apply_convrule.h"
 #include "qs_parse_string.h"
 
 #include "http_core.h"
+
+#if 1
 #include <wand/magick_wand.h>
+#endif
+
 
 #define EXIT_MAGICK_ERROR() \
   do { \
@@ -57,7 +72,7 @@ typedef struct _query_string_param_t {
   char*             user_agent;
   ua_use_flag_t     ua_flag;
 
-  char*             name;     /* for EZGET */
+  char*             name;      /* for EZGET */
   long              offset;    /* for EZGET */
   long              count;     /* for EZGET */
   int               width;
@@ -167,6 +182,11 @@ static apr_status_t s_send_cache_file(  device_table* spec,
                                         query_string_param_t* query_string,
                                         request_rec* r,
                                         const char* tmpfile);
+static apr_status_t s_header_only_cache_file(
+  device_table* spec, 
+  query_string_param_t* query_string, 
+  request_rec* r, 
+  const char* tmpfile);
 static query_string_param_t* s_get_query_string_param(request_rec *r);
 static unsigned short s_add_crc(        const char* writedata, 
                                         apr_size_t witebyte);
@@ -206,41 +226,59 @@ chxj_img_conv_format_handler(request_rec* r)
   query_string_param_t* qsp;
   char*                 user_agent;
   device_table*       spec;
+  chxjconvrule_entry* entryp;
+
+  DBG(r, "start chxj_img_conv_format_handler()");
   
   if ((*r->handler != 'c' && *r->handler != 'C') 
   ||  (strcasecmp(r->handler, "chxj-picture")
-  &&  strcasecmp(r->handler, "chxj-qrcode")))
+  &&  strcasecmp(r->handler, "chxj-qrcode"))) {
+    DBG(r, "end chxj_img_conv_format_handler()");
     return DECLINED;
+  }
 
   qsp = s_get_query_string_param(r);
   conf = ap_get_module_config(r->per_dir_config, &chxj_module);
+  if (conf == NULL) {
+    DBG(r, "end chxj_img_conv_format_handler() conf is null");
+    return DECLINED;
+  }
 
-  if (strcasecmp(r->handler, "chxj-qrcode") == 0 &&  conf->image == CHXJ_IMG_OFF)
+  if (strcasecmp(r->handler, "chxj-qrcode") == 0 &&  conf->image == CHXJ_IMG_OFF) 
     return DECLINED;
 
-  if (r->header_only) 
-    return DECLINED;
 
+  /*------------------------------------------------------------------------*/
+  /* get UserAgent from http header                                         */
+  /*------------------------------------------------------------------------*/
   /*--------------------------------------------------------------------------*/
   /* User-Agent to spec                                                       */
   /*--------------------------------------------------------------------------*/
-  if (qsp->user_agent)
+  if (qsp->user_agent) {
     user_agent = apr_pstrdup(r->pool, qsp->user_agent);
-  else
-    user_agent = (char*)apr_table_get(r->headers_in, HTTP_USER_AGENT);
+  }
+  else {
+    entryp = chxj_apply_convrule(r, conf->convrules);
+    if (entryp && entryp->user_agent) {
+      user_agent = (char*)apr_table_get(r->headers_in, "CHXJ_HTTP_USER_AGENT");
+    }
+    else {
+      user_agent = (char*)apr_table_get(r->headers_in, "User-Agent");
+    }
+  }
+
+
 
   if (qsp->ua_flag == UA_IGN)
     spec = &v_ignore_spec;
   else
     spec = chxj_specified_device(r, user_agent);
 
-  ap_log_rerror(APLOG_MARK,APLOG_DEBUG, 0, r, "found device_name=[%s]", 
-                  spec->device_name);
-  ap_log_rerror(APLOG_MARK,APLOG_DEBUG, 0, r, "User-Agent=[%s]", 
-                  user_agent);
+  DBG1(r,"found device_name=[%s]", spec->device_name);
+  DBG1(r,"User-Agent=[%s]", user_agent);
 
   if (spec->width == 0 || spec->heigh == 0)
-    return HTTP_NOT_FOUND;
+    return DECLINED;
 
   return s_img_conv_format_from_file(r, conf, user_agent, qsp, spec);
 }
@@ -260,20 +298,42 @@ chxj_exchange_image(request_rec *r, const char** src, apr_size_t* len)
   mod_chxj_config*      conf;
   query_string_param_t* qsp;
   char*                 user_agent;
-  device_table*       spec;
+  device_table*         spec;
   char*                 dst;
+  char*                 conv_check;
+  chxjconvrule_entry* entryp;
+
+  DBG(r, "start chxj_exchange_image()");
+
+  conv_check = (char*)apr_table_get(r->headers_in, "CHXJ_IMG_CONV");
+  if (conv_check) {
+    DBG(r, "end chxj_exchnage_image() already convert.");
+    return NULL;
+  }
 
 
   qsp = s_get_query_string_param(r);
   conf = ap_get_module_config(r->per_dir_config, &chxj_module);
+  if (conf == NULL) {
+    DBG(r, "end chxj_exchange_image()");
+    return NULL;
+  }
 
   /*--------------------------------------------------------------------------*/
   /* User-Agent to spec                                                       */
   /*--------------------------------------------------------------------------*/
-  if (qsp->user_agent != NULL)
+  if (qsp->user_agent) {
     user_agent = apr_pstrdup(r->pool, qsp->user_agent);
-  else
-    user_agent = (char*)apr_table_get(r->headers_in, HTTP_USER_AGENT);
+  }
+  else {
+    entryp = chxj_apply_convrule(r, conf->convrules);
+    if (entryp && entryp->user_agent) {
+      user_agent = (char*)apr_table_get(r->headers_in, "CHXJ_HTTP_USER_AGENT");
+    }
+    else {
+      user_agent = (char*)apr_table_get(r->headers_in, "User-Agent");
+    }
+  }
 
   if (qsp->ua_flag == UA_IGN)
     spec = &v_ignore_spec;
@@ -290,6 +350,8 @@ chxj_exchange_image(request_rec *r, const char** src, apr_size_t* len)
   if (dst == NULL) 
     *len = 0;
 
+  DBG(r, "end chxj_exchange_image()");
+
   return dst;
 }
 
@@ -332,9 +394,17 @@ s_img_conv_format_from_file(
   }
 
   DBG1(r,"color=[%d]", spec->color);
-  rv = s_send_cache_file(spec, qsp,r, tmpfile);
-  if (rv != OK) 
-    return rv;
+  if (! r->header_only)  {
+    rv = s_send_cache_file(spec, qsp,r, tmpfile);
+    if (rv != OK) 
+      return rv;
+  }
+  else {
+    rv = s_header_only_cache_file(spec, qsp, r, tmpfile);
+    if (rv != OK) 
+      return rv;
+  }
+  apr_table_setn(r->headers_in, "CHXJ_IMG_CONV", "done");
 
   DBG(r,"end chxj_img_conv_format");
 
@@ -918,11 +988,11 @@ s_fixup_size(MagickWand* magick_wand,
 
     case IMG_CONV_MODE_NORMAL:
       if (qsp->width) {
-        ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,"convert width=[%d --> %d]", neww, qsp->width);
+        DBG2(r,"convert width=[%d --> %d]", neww, qsp->width);
         neww = qsp->width;
       }
       if (qsp->height) {
-        ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,"convert heigh=[%d --> %d]", newh, qsp->height);
+        DBG2(r,"convert heigh=[%d --> %d]", newh, qsp->height);
         newh = qsp->height;
       }
 
@@ -1331,6 +1401,86 @@ s_send_cache_file(device_table* spec, query_string_param_t* query_string, reques
   return OK;
 }
 
+static apr_status_t 
+s_header_only_cache_file(device_table* spec, query_string_param_t* query_string, request_rec* r, const char* tmpfile)
+{
+  apr_status_t rv;
+  apr_finfo_t  st;
+  char*        contentLength;
+
+  rv = apr_stat(&st, tmpfile, APR_FINFO_MIN, r->pool);
+  if (rv != APR_SUCCESS)
+    return HTTP_NOT_FOUND;
+
+  DBG1(r, "mode:[%d]",    query_string->mode);
+  DBG1(r, "name:[%s]",    query_string->name);
+  DBG1(r, "offset:[%ld]", query_string->offset);
+  DBG1(r, "count:[%ld]",  query_string->count);
+
+  if (spec->available_jpeg) {
+    r->content_type = apr_psprintf(r->pool, "image/jpeg");
+  }
+  else
+  if (spec->available_png) {
+    r->content_type = apr_psprintf(r->pool, "image/png");
+  }
+  else
+  if (spec->available_gif) {
+    r->content_type = apr_psprintf(r->pool, "image/gif");
+  }
+  else
+  if (spec->available_bmp2 || spec->available_bmp4) {
+    r->content_type = apr_psprintf(r->pool, "image/bmp");
+  }
+
+  if (query_string->mode != IMG_CONV_MODE_EZGET && query_string->name == NULL) {
+    contentLength = apr_psprintf(r->pool, "%d", (int)st.size);
+    apr_table_setn(r->headers_out, "Content-Length", (const char*)contentLength);
+  
+    DBG1(r,"Content-Length:[%d]", (int)st.size);
+  }
+  else
+  if (query_string->mode == IMG_CONV_MODE_EZGET) {
+    char* name = apr_pstrdup(r->pool, basename(r->filename));
+    name[strlen(name)-4] = 0;
+    if (strcasecmp(r->content_type, "image/jpeg") == 0) {
+
+      ap_set_content_type(r, "text/x-hdml; charset=Shift_JIS");
+    }
+    else
+    if (strcasecmp(r->content_type, "image/bmp") == 0) {
+      ap_set_content_type(r, "text/x-hdml; charset=Shift_JIS");
+    }
+    else
+    if (strcasecmp(r->content_type, "image/png") == 0) {
+      ap_set_content_type(r, "text/x-hdml; charset=Shift_JIS");
+    }
+    else
+    if (strcasecmp(r->content_type, "image/gif") == 0) {
+      ap_set_content_type(r, "text/x-hdml; charset=Shift_JIS");
+    }
+  }
+  else
+  if (query_string->mode == IMG_CONV_MODE_WALLPAPER && query_string->name != NULL) {
+    if (query_string->count == -1 && query_string->offset == -1) {
+      ap_set_content_type(r, "text/x-hdml; charset=Shift_JIS");
+    }
+    else
+    if (query_string->count == -2 && query_string->offset == -1) {
+      ap_set_content_type(r, "text/x-hdml; charset=Shift_JIS");
+    }
+    else { 
+      ap_set_content_type(r, "application/x-up-download");
+      contentLength = apr_psprintf(r->pool, "%ld", query_string->count);
+      apr_table_setn(r->headers_out, "Content-Length", (const char*)contentLength);
+  
+      DBG1(r, "Content-Length:[%d]", (int)st.size);
+    }
+  }
+  
+  return OK;
+}
+
 
 
 static char*
@@ -1419,19 +1569,33 @@ chxj_trans_name(request_rec *r)
   int ii;
   char*      ext[] = {
           "jpg",
+          "jpeg",
           "png",
           "bmp",
           "gif",
           "qrc",    /* QRCodeѥեγĥ */
+          "",
   };
   char*    fname;
   char*    idx;
   char*    filename_sv;
+  int      do_ext_check = TRUE;
+  int      next_ok      = FALSE;
+
+  DBG(r, "start chxj_trans_name()");
 
   conf = ap_get_module_config(r->per_dir_config, &chxj_module);
 
-  if (conf->image != CHXJ_IMG_ON) 
+  if (conf == NULL) {
+    DBG1(r, "end chxj_trans_name() conf is null[%s]", r->uri);
     return DECLINED;
+  }
+
+  if (conf->image != CHXJ_IMG_ON) {
+    DBG(r, "end chxj_trans_name() conf not found");
+    return DECLINED;
+  }
+
 
   DBG1(r,"Match URI[%s]", r->uri);
 
@@ -1467,21 +1631,55 @@ chxj_trans_name(request_rec *r)
 
   DBG1(r,"URI[%s]", filename_sv);
 
+  do_ext_check = TRUE;
+  for (ii=0; ii<7-1; ii++) {
+    char* pos = strrchr(filename_sv, '.');
+    if (pos && pos++) {
+      if (strcasecmp(pos, ext[ii]) == 0) {
+        do_ext_check = FALSE;
+        fname = apr_psprintf(r->pool, "%s", filename_sv);
+        break;
+      }
+    }
+  }
 
-  for (ii=0; ii<5; ii++) {
-    fname = apr_psprintf(r->pool, "%s.%s", filename_sv, ext[ii]);
-    DBG1(r,"search [%s]", fname);
-
-    rv = apr_stat(&st, fname, APR_FINFO_MIN, r->pool);
-    if (rv == APR_SUCCESS)
-      break;
-
-    fname = NULL;
+  if (do_ext_check) {
+    for (ii=0; ii<7; ii++) {
+      if (strlen(ext[ii]) == 0) {
+        fname = apr_psprintf(r->pool, "%s", filename_sv);
+      }
+      else 
+        fname = apr_psprintf(r->pool, "%s.%s", filename_sv, ext[ii]);
+  
+      DBG1(r,"search [%s]", fname);
+  
+      rv = apr_stat(&st, fname, APR_FINFO_MIN, r->pool);
+      if (rv == APR_SUCCESS) {
+        if (st.filetype != APR_DIR)
+          break;
+      }
+  
+      fname = NULL;
+    }
   }
   if (fname == NULL) {
     DBG1(r,"NotFound [%s]", r->filename);
     return DECLINED;
   }
+  for (ii=0; ii<7-1; ii++) {
+    char* pos = strrchr(fname, '.');
+    if (pos && pos++) {
+      if (strcasecmp(pos, ext[ii]) == 0) {
+        next_ok = TRUE;
+        break;
+      }
+    }
+  }
+
+  if (! next_ok)  {
+    DBG1(r,"NotFound [%s]", r->filename);
+    return DECLINED;
+  }
 
   if (r->handler == NULL || strcasecmp(r->handler, "chxj-qrcode") != 0) {
     DBG1(r,"Found [%s]", fname);
@@ -1493,6 +1691,7 @@ chxj_trans_name(request_rec *r)
     else
       r->handler = apr_psprintf(r->pool, "chxj-picture");
   }
+  DBG(r, "end chxj_trans_name()");
   return OK;
 }
 
