@@ -14,6 +14,15 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
+#if 0
+#include <wand/magick_wand.h>
+
+#undef PACKAGE_BUGREPORT
+#undef PACKAGE_NAME
+#undef PACKAGE_STRING
+#undef PACKAGE_TARNAME
+#undef PACKAGE_VERSION
+#endif
 #include <unistd.h>
 #include <assert.h>
 #include "apr_strings.h"
@@ -602,7 +611,7 @@ chxj_qr_code_handler(request_rec* r)
   /* ⤷᡼ѴϥɥǤСǤϽʤ                 */
   /*--------------------------------------------------------------------------*/
   conf = ap_get_module_config(r->per_dir_config, &chxj_module);
-  if (conf->image == CHXJ_IMG_ON)
+  if (!conf || conf->image == CHXJ_IMG_ON)
     return DECLINED;
 
   memset(&doc, 0, sizeof(Doc));
@@ -873,7 +882,7 @@ chxj_qr_code(qr_code_t* qrcode, char* module[])
   }
 #ifdef QR_CODE_DEBUG
   DBG3(r, "After TERM BIT[%s] len[%d]bit [%f]byte", binstr, strlen(binstr), (float)(((float)strlen(binstr))/8.0));
-  ap_log_rerror(APLOG_MARK,APLOG_DEBUG, 0, r, "add term data");
+  DBG(r,"add term data");
 #endif
 
   /*--------------------------------------------------------------------------*/
