@@ -36,19 +36,19 @@ qs_get_value_attr(Doc* doc, Node* node, request_rec* r)
   /* The object tag node is scanned.                                          */
   /*--------------------------------------------------------------------------*/
   for (attr = qs_get_attr(doc,node);
-       attr != NULL;
-       attr = qs_get_next_attr(doc,attr))
-  {
+       attr;
+       attr = qs_get_next_attr(doc,attr)) {
+
     char* name  = qs_get_attr_name(doc,attr);
     char* value = qs_get_attr_value(doc,attr);
-    if (strcasecmp(name, "value") == 0)
-    {
+
+    if ((*name == 'v' || *name == 'V') && strcasecmp(name, "value") == 0)
       /*----------------------------------------------------------------------*/
       /* The VALUE attribute was found.                                       */
       /*----------------------------------------------------------------------*/
       return apr_pstrdup(r->pool, value);
-    }
   }
+
   /*--------------------------------------------------------------------------*/
   /* not found                                                                */
   /*--------------------------------------------------------------------------*/
@@ -76,11 +76,9 @@ qs_get_checked_attr(Doc* doc, Node* tag, request_rec* r)
   /*--------------------------------------------------------------------------*/
   for (attr = qs_get_attr(doc,tag);
        attr != NULL;
-       attr = qs_get_next_attr(doc,attr))
-  {
+       attr = qs_get_next_attr(doc,attr)) {
     char* name  = qs_get_attr_name(doc,attr);
-    if (strcasecmp(name, "checked") == 0)
-    {
+    if ((*name == 'c' || *name == 'C') && strcasecmp(name, "checked") == 0) {
       /*----------------------------------------------------------------------*/
       /* The VALUE attribute was found.                                       */
       /*----------------------------------------------------------------------*/
@@ -88,15 +86,16 @@ qs_get_checked_attr(Doc* doc, Node* tag, request_rec* r)
       break;
     }
   }
-  if (found_flag == 0)
-  {
+  if (! found_flag)
     /*------------------------------------------------------------------------*/
     /* not found                                                              */
     /*------------------------------------------------------------------------*/
     return NULL;
-  }
+
   return qs_get_value_attr(doc, tag, r);
 }
+
+
 /**
  * The value of the type attribute is acquired.
  *
@@ -117,18 +116,16 @@ qs_get_type_attr(Doc* doc, Node* tag, request_rec* r)
   /*--------------------------------------------------------------------------*/
   for (attr = qs_get_attr(doc,tag);
        attr != NULL;
-       attr = qs_get_next_attr(doc,attr))
-  {
+       attr = qs_get_next_attr(doc,attr)) {
+
     char* name  = qs_get_attr_name(doc,attr);
     char* value = qs_get_attr_value(doc,attr);
 
-    if (strcasecmp(name, "type") == 0)
-    {
+    if ((*name == 't' || *name == 'T') && strcasecmp(name, "type") == 0) 
       /*----------------------------------------------------------------------*/
       /* The VALUE attribute was found.                                       */
       /*----------------------------------------------------------------------*/
       return apr_pstrdup(r->pool, value);
-    }
   }
   /*--------------------------------------------------------------------------*/
   /* not found                                                                */
@@ -196,45 +193,40 @@ qs_get_selected_value_text(Doc *doc, Node* node, request_rec* r)
 
   for (child = qs_get_child_node(doc,node);
        child != NULL; 
-       child = qs_get_next_node(doc,child))
-  {
+       child = qs_get_next_node(doc,child)) {
+
     char* name = qs_get_node_name(doc,child);
 
     /*------------------------------------------------------------------------*/
     /* <OPTION> tag                                                           */
     /*------------------------------------------------------------------------*/
-    if (strcasecmp(name, "option") == 0) 
-    {
+    if ((*name == 'o' || *name == 'O') && strcasecmp(name, "option") == 0) {
       Attr* attr;
+
       for (attr =  qs_get_attr(doc,child); 
            attr != NULL; 
-           attr = qs_get_next_attr(doc,attr)) 
-      {
+           attr = qs_get_next_attr(doc,attr)) {
+
         char* name  = qs_get_attr_name(doc,attr);
 
-        ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, 
-                        "qs_get_selected_value name::[%s]" , name);
-        if (strcasecmp(name, "selected") == 0) 
-        {
+        DBG1(r, "qs_get_selected_value name::[%s]" , name);
+
+        if ((*name == 's'|| *name == 'S') && strcasecmp(name, "selected") == 0) {
           /*------------------------------------------------------------------*/
           /* SELECTED Value Found                                             */
           /*------------------------------------------------------------------*/
           selchild = qs_get_child_node(doc, child);
-          if (selchild == NULL) 
-          {
-            ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, 
-                            "found selected tag but null node" );
+          if (! selchild) {
+            DBG(r,"found selected tag but null node" );
             return NULL;
           }
           return qs_get_node_value(doc, selchild);
         }
       }
     }
-    result = qs_get_selected_value_text(doc, child, r);
-    if (result != NULL)
-    {
+
+    if ((result = qs_get_selected_value_text(doc, child, r)) != NULL)
       return result;
-    }
   }
 
   /*--------------------------------------------------------------------------*/
@@ -261,38 +253,34 @@ qs_get_selected_value(Doc* doc, Node* node, request_rec* r)
 
   for (child = qs_get_child_node(doc,node); 
        child != NULL; 
-       child = qs_get_next_node(doc,child))
-  {
+       child = qs_get_next_node(doc,child)) {
+
     char* name = qs_get_node_name(doc,child);
 
     /*------------------------------------------------------------------------*/
     /* <OPTION> tag                                                           */
     /*------------------------------------------------------------------------*/
-    if (strcasecmp(name, "option") == 0) 
-    {
+    if ((*name == 'o' || *name == 'O') && strcasecmp(name, "option") == 0) {
       Attr* attr;
+
       for (attr = qs_get_attr(doc,child); 
-           attr != NULL; 
-           attr = qs_get_next_attr(doc,attr)) 
-      {
+           attr; 
+           attr = qs_get_next_attr(doc,attr)) {
+
         char* name  = qs_get_attr_name(doc,attr);
 
-        ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, 
-                        "qs_get_selected_value name::[%s]" , name);
-        if (strcasecmp(name, "selected") == 0) 
-        {
+        DBG1(r, "qs_get_selected_value name::[%s]" , name);
+
+        if ((*name == 's' || *name == 'S') && strcasecmp(name, "selected") == 0)
           /*------------------------------------------------------------------*/
           /* SELECTED Value Found                                             */
           /*------------------------------------------------------------------*/
           return qs_get_value_attr(doc, child, r);
-        }
       }
     }
-    result = qs_get_selected_value(doc, child, r);
-    if (result != NULL)
-    {
+
+    if ((result = qs_get_selected_value(doc, child, r)) != NULL)
       return result;
-    }
   }
 
   /*--------------------------------------------------------------------------*/
@@ -317,15 +305,15 @@ qs_get_name_attr(Doc* doc, Node* tag, request_rec* r)
 {
   Attr* attr;
 
-  for (attr = qs_get_attr(doc,tag); attr != NULL; attr = qs_get_next_attr(doc,attr))
-  {
+  for (attr = qs_get_attr(doc,tag); 
+       attr; 
+       attr = qs_get_next_attr(doc,attr)) {
+
     char* name  = qs_get_attr_name(doc,attr);
     char* value = qs_get_attr_value(doc,attr);
 
-    if (strcasecmp(name, "name") == 0)
-    {
+    if ((*name == 'n' || *name == 'N') && strcasecmp(name, "name") == 0)
       return apr_pstrdup(r->pool, value);
-    }
   }
 
   return NULL;
@@ -347,15 +335,15 @@ qs_get_size_attr(Doc* doc, Node* tag, request_rec* r)
 {
   Attr* attr;
 
-  for (attr = qs_get_attr(doc,tag); attr != NULL; attr = qs_get_next_attr(doc,attr))
-  {
+  for (attr = qs_get_attr(doc,tag); 
+       attr; 
+       attr = qs_get_next_attr(doc,attr)) {
+
     char* name  = qs_get_attr_name(doc,attr);
     char* value = qs_get_attr_value(doc,attr);
 
-    if (strcasecmp(name, "size") == 0)
-    {
+    if ((*name == 's' || *name == 'S') && strcasecmp(name, "size") == 0)
       return apr_pstrdup(r->pool, value);
-    }
   }
 
   return NULL;
@@ -377,15 +365,15 @@ qs_get_accesskey_attr(Doc* doc, Node* tag, request_rec* r)
 {
   Attr* attr;
 
-  for (attr = qs_get_attr(doc,tag); attr != NULL; attr = qs_get_next_attr(doc,attr))
-  {
+  for (attr = qs_get_attr(doc,tag); 
+       attr; 
+       attr = qs_get_next_attr(doc,attr)) {
+
     char* name  = qs_get_attr_name(doc,attr);
     char* value = qs_get_attr_value(doc,attr);
 
-    if (strcasecmp(name, "accesskey") == 0)
-    {
+    if ((*name == 'a' || *name == 'A') && strcasecmp(name, "accesskey") == 0)
       return apr_pstrdup(r->pool, value);
-    }
   }
 
   return NULL;
@@ -409,15 +397,13 @@ qs_get_istyle_attr(Doc* doc, Node* tag, request_rec* r)
 
   for (attr = qs_get_attr(doc,tag); 
        attr != NULL; 
-       attr = qs_get_next_attr(doc,attr))
-  {
+       attr = qs_get_next_attr(doc,attr)) {
+
     char* name  = qs_get_attr_name(doc,attr);
     char* value = qs_get_attr_value(doc,attr);
 
-    if (strcasecmp(name, "istyle") == 0)
-    {
+    if ((*name == 'i' || *name == 'I') && strcasecmp(name, "istyle") == 0)
       return apr_pstrdup(r->pool, value);
-    }
   }
 
   return NULL;
@@ -442,16 +428,15 @@ qs_get_maxlength_attr(Doc* doc, Node* tag, request_rec* r)
 
   for (attr = qs_get_attr(doc,tag);
        attr != NULL; 
-       attr = qs_get_next_attr(doc,attr))
-  {
+       attr = qs_get_next_attr(doc,attr)) {
+
     char* name  = qs_get_attr_name(doc,attr);
     char* value = qs_get_attr_value(doc,attr);
 
-    if (strcasecmp(name, "maxlength") == 0)
-    {
+    if ((*name == 'm' || *name == 'M') && strcasecmp(name, "maxlength") == 0)
       return apr_pstrdup(r->pool, value);
-    }
   }
+
   return NULL;
 }
 
@@ -471,65 +456,58 @@ qs_is_checked_checkbox_attr(Doc* doc, Node* tag, request_rec* r)
   Attr* attr;
 
   for (attr = qs_get_attr(doc,tag);
-       attr != NULL; 
-       attr = qs_get_next_attr(doc,attr))
-  {
+       attr; 
+       attr = qs_get_next_attr(doc,attr)) {
+
     char* name  = qs_get_attr_name(doc,attr);
 
-    if (strcasecmp(name, "checked") == 0)
-    {
+    if ((*name == 'c' || *name == 'C') && strcasecmp(name, "checked") == 0)
       return 1;
-    }
   }
+
   return 0;
 }
 
 
 int
-chxj_chxjif_is_mine(device_table_t* spec, Doc* doc, Node* tag)
+chxj_chxjif_is_mine(device_table* spec, Doc* doc, Node* tag)
 {
   request_rec* r = doc->r;
   Attr* attr;
 
   for (attr = qs_get_attr(doc,tag);
-       attr != NULL; 
-       attr = qs_get_next_attr(doc,attr))
-  {
+       attr; 
+       attr = qs_get_next_attr(doc,attr)) {
+
     char* name  = qs_get_attr_name(doc,attr);
     char* value = qs_get_attr_value(doc,attr);
 
-    if (strcasecmp(name, "lang") == 0)
-    {
-      ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "lang found [%s] spec [%d]", value, spec->html_spec_type);
-      if (strcasecmp(value, "xhtml") == 0) 
-      {
-        if (spec->html_spec_type == CHXJ_SPEC_XHtml_Mobile_1_0)
-        {
+    if ((*name == 'l' || *name == 'L') && strcasecmp(name, "lang") == 0) {
+
+      DBG2(r, "lang found [%s] spec [%d]", value, spec->html_spec_type);
+
+      if ((*value == 'x' || *value == 'X') && strcasecmp(value, "xhtml") == 0) {
+        if (spec->html_spec_type == CHXJ_SPEC_XHtml_Mobile_1_0) {
           /* Yes , it is mine */
           return 1;
         }
       }
       else
-      if (strcasecmp(value, "hdml") == 0)
-      {
-        if (spec->html_spec_type == CHXJ_SPEC_Hdml)
-        {
+      if ((*value == 'h' || *value == 'H') && strcasecmp(value, "hdml") == 0) {
+        if (spec->html_spec_type == CHXJ_SPEC_Hdml) {
           /* Yes , it is mine */
           return 1;
         }
       }
       else
-      if (strcasecmp(value, "jhtml") == 0)
-      {
-        if (spec->html_spec_type == CHXJ_SPEC_Jhtml)
-        {
+      if ((*value == 'j' || *value == 'J') && strcasecmp(value, "jhtml") == 0) {
+        if (spec->html_spec_type == CHXJ_SPEC_Jhtml) {
           /* Yes , it is mine */
           return 1;
         }
       }
       else
-      if (strcasecmp(value, "chtml") == 0)
-      {
+      if ((*value == 'c' || *value == 'C') && strcasecmp(value, "chtml") == 0) {
         switch (spec->html_spec_type) {
         case CHXJ_SPEC_Chtml_1_0:
         case CHXJ_SPEC_Chtml_2_0:
@@ -566,19 +544,20 @@ qs_get_destlang_attr(Doc* doc, Node* tag, request_rec* r)
   Attr*        attr;
 
   for (attr = qs_get_attr(doc,tag);
-       attr != NULL; 
-       attr = qs_get_next_attr(doc,attr))
-  {
+       attr; 
+       attr = qs_get_next_attr(doc,attr)) {
+
     char* name  = qs_get_attr_name(doc,attr);
     char* value = qs_get_attr_value(doc,attr);
 
-    if (strcasecmp(name, "destlang") == 0)
-    {
+    if ((*name == 'd' || *name == 'D') && strcasecmp(name, "destlang") == 0)
       return apr_pstrdup(r->pool, value);
-    }
   }
+
   return NULL;
 }
+
+
 /**
  * The value of the PARSE attribute is acquired.
  *
@@ -598,20 +577,20 @@ qs_get_parse_attr(Doc* doc, Node* tag, request_rec* r)
   /* The object tag node is scanned.                                          */
   /*--------------------------------------------------------------------------*/
   for (attr = qs_get_attr(doc,tag);
-       attr != NULL;
-       attr = qs_get_next_attr(doc,attr))
-  {
+       attr;
+       attr = qs_get_next_attr(doc,attr)) {
+
     char* name  = qs_get_attr_name(doc,attr);
     char* value = qs_get_attr_value(doc,attr);
 
-    if (strcasecmp(name, "parse") == 0)
-    {
+    if ((*name == 'p' || *name == 'P') && strcasecmp(name, "parse") == 0) {
       /*----------------------------------------------------------------------*/
       /* The VALUE attribute was found.                                       */
       /*----------------------------------------------------------------------*/
       return apr_pstrdup(r->pool, value);
     }
   }
+
   /*--------------------------------------------------------------------------*/
   /* not found                                                                */
   /*--------------------------------------------------------------------------*/
