@@ -61,6 +61,106 @@
 #define CHXJ_VERSION_PREFIX PACKAGE_NAME "/"
 #define CHXJ_VERSION        PACKAGE_VERSION
 
+exchange_t exchange_routine[] = {
+  {
+    /* CHXJ_SPEC_UNKNOWN          */
+    .exchange = NULL,
+    .encoder  = NULL,
+  },
+  {
+    /* CHXJ_SPEC_Chtml_1_0        */
+    .exchange = chxj_exchange_chtml10,
+    .encoder  = chxj_encoding,
+  },
+  {
+    /* CHXJ_SPEC_Chtml_2_0        */
+    .exchange = chxj_exchange_chtml20,
+    .encoder  = chxj_encoding,
+  },
+  {
+    /* CHXJ_SPEC_Chtml_3_0        */
+    .exchange = chxj_exchange_chtml30,
+    .encoder  = chxj_encoding,
+  },
+  {
+    /* CHXJ_SPEC_Chtml_4_0        */
+    .exchange = chxj_exchange_chtml30,
+    .encoder  = chxj_encoding,
+  },
+  {
+    /* CHXJ_SPEC_Chtml_5_0        */
+    .exchange = chxj_exchange_chtml30,
+    .encoder  = chxj_encoding,
+  },
+  {
+    /* CHXJ_SPEC_XHtml_Mobile_1_0 */
+    .exchange = chxj_exchange_xhtml_mobile_1_0,
+    .encoder  = chxj_encoding,
+  },
+  {
+    /* CHXJ_SPEC_Hdml             */
+    .exchange = chxj_exchange_hdml,
+    .encoder  = chxj_encoding,
+  },
+  {
+    /* CHXJ_SPEC_Jhtml            */
+    .exchange = chxj_exchange_jhtml,
+    .encoder  = chxj_encoding,
+  },
+  {
+    /* CHXJ_SPEC_HTML             */
+    .exchange = NULL,
+    .encoder  = NULL,
+  },
+  {
+    NULL
+  }
+};
+
+static apr_status_t 
+chxj_headers_fixup(request_rec *r)
+{
+  mod_chxj_config*    dconf; 
+  chxjconvrule_entry* entryp;
+  char*               user_agent;
+  device_table*       spec;
+
+  DBG(r, "start chxj_headers_fixup()");
+  dconf = ap_get_module_config(r->per_dir_config, &chxj_module);
+
+  user_agent = (char*)apr_table_get(r->headers_in, "User-Agent");
+  spec = chxj_specified_device(r, user_agent);
+
+  switch(spec->html_spec_type) {
+  case CHXJ_SPEC_Chtml_1_0:
+  case CHXJ_SPEC_Chtml_2_0:
+  case CHXJ_SPEC_Chtml_3_0:
+  case CHXJ_SPEC_Chtml_4_0:
+  case CHXJ_SPEC_Chtml_5_0:
+  case CHXJ_SPEC_XHtml_Mobile_1_0:
+  case CHXJ_SPEC_Hdml:
+  case CHXJ_SPEC_Jhtml:
+    entryp = chxj_apply_convrule(r, dconf->convrules);
+    if (! entryp) {
+      DBG(r, "end chxj_headers_fixup() no pattern");
+      return DECLINED;
+    }
+  
+    apr_table_setn(r->headers_in, "CHXJ_HTTP_USER_AGENT", user_agent);
+  
+    if (entryp->user_agent)
+      apr_table_setn(r->headers_in, "User-Agent", entryp->user_agent);
+    break;
+  
+  default:
+    break;
+  }
+
+  DBG(r, "end chxj_headers_fixup()");
+
+  return DECLINED;
+}
+
 /**
  * It converts it from CHTML into XXML corresponding to each model. 
  *
@@ -79,22 +179,28 @@ chxj_exchange(request_rec *r, const char** src, apr_size_t* len)
 
   dconf = ap_get_module_config(r->per_dir_config, &chxj_module);
 
-
-  /*------------------------------------------------------------------------*/
-  /* get UserAgent from http header                                         */
-  /*------------------------------------------------------------------------*/
-  user_agent = (char*)apr_table_get(r->headers_in, HTTP_USER_AGENT);
-
-  DBG1(r,"User-Agent:[%s]", user_agent);
-  DBG(r, "start chxj_exchange()");
-  DBG1(r,"content type is %s", r->content_type);
-
   entryp = chxj_apply_convrule(r, dconf->convrules);
   if (!(entryp->action & CONVRULE_ENGINE_ON_BIT)) {
     DBG(r,"EngineOff");
     return (char*)*src;
   }
 
+
+  /*------------------------------------------------------------------------*/
+  /* get UserAgent from http header                                         */
+  /*------------------------------------------------------------------------*/
+  if (entryp->user_agent) {
+    user_agent = (char*)apr_table_get(r->headers_in, "CHXJ_HTTP_USER_AGENT");
+  }
+  else {
+    user_agent = (char*)apr_table_get(r->headers_in, "User-Agent");
+  }
+
+  DBG1(r,"User-Agent:[%s]", user_agent);
+  DBG(r, "start chxj_exchange()");
+  DBG1(r,"content type is %s", r->content_type);
+
+
   if (*(char*)r->content_type == 't' 
   && strncmp(r->content_type, "text/html",   9) != 0) {
     DBG1(r,"content type is %s", r->content_type);
@@ -104,83 +210,15 @@ chxj_exchange(request_rec *r, const char** src, apr_size_t* len)
   if (!r->header_only) {
     device_table* spec = chxj_specified_device(r, user_agent);
 
-    switch(spec->html_spec_type) {
-    case CHXJ_SPEC_Chtml_1_0:
-      /*----------------------------------------------------------------------*/
-      /* DoCoMo i-Mode 1.0                                                    */
-      /*----------------------------------------------------------------------*/
-      DBG(r,"select DoCoMo i-Mode 1.0 ");
-      tmp = chxj_encoding(r, *src, (apr_size_t*)len);
-      dst = chxj_exchange_chtml10(r, spec, tmp, *len, len, entryp);
-      break;
+    tmp = NULL;
+    if (exchange_routine[spec->html_spec_type].encoder)
+      tmp = exchange_routine[spec->html_spec_type].encoder(r, *src, (apr_size_t*)len);
 
-    case CHXJ_SPEC_Chtml_2_0:
-      /*----------------------------------------------------------------------*/
-      /* DoCoMo i-Mode 2.0                                                    */
-      /*----------------------------------------------------------------------*/
-      DBG(r,"select DoCoMo i-Mode 2.0 ");
-      tmp = chxj_encoding(r, (char*)*src, (apr_size_t*)len);
-      dst = chxj_exchange_chtml20(r, spec, tmp, *len, len, entryp);
-      break;
-   
-    case CHXJ_SPEC_Chtml_3_0:
-      /*----------------------------------------------------------------------*/
-      /* DoCoMo i-Mode 3.0                                                    */
-      /*----------------------------------------------------------------------*/
-      DBG(r,"select DoCoMo i-Mode 3.0 ");
-      tmp = chxj_encoding(r, *src, (apr_size_t*)len);
-      dst = chxj_exchange_chtml30(r, spec, tmp, *len, len, entryp);
-      break;
-    
-    case CHXJ_SPEC_Chtml_4_0:
-      /*----------------------------------------------------------------------*/
-      /* DoCoMo i-Mode 4.0                                                    */
-      /*----------------------------------------------------------------------*/
-      DBG(r,"select DoCoMo i-Mode 4.0 ");
-      tmp = chxj_encoding(r, *src, (apr_size_t*)len);
-      dst = chxj_exchange_chtml30(r, spec, tmp, *len, len, entryp);
-      break;
-
-    case CHXJ_SPEC_Chtml_5_0:
-      /*----------------------------------------------------------------------*/
-      /* DoCoMo i-Mode 5.0                                                    */
-      /*----------------------------------------------------------------------*/
-      DBG(r,"select DoCoMo i-Mode 5.0 ");
-      tmp = chxj_encoding(r, *src, (apr_size_t*)len);
-      dst = chxj_exchange_chtml30(r, spec, tmp, *len, len, entryp);
-      break;
-
-    case CHXJ_SPEC_XHtml_Mobile_1_0:
-      /*----------------------------------------------------------------------*/
-      /* AU XHtml Mobile 1.0 (XHtml Basic 1.0 extended)                       */
-      /*----------------------------------------------------------------------*/
-      DBG(r,"select XHTML Mobile 1.0");
-      tmp = chxj_encoding(r, *src, (apr_size_t*)len);
-      dst = chxj_exchange_xhtml_mobile_1_0(r, spec, tmp, *len, len, entryp);
-      break;
-
-    case CHXJ_SPEC_Hdml:
-      /*----------------------------------------------------------------------*/
-      /* AU HDML Version 3.0 Only                                             */
-      /*----------------------------------------------------------------------*/
-      DBG(r,"select HDML");
-      tmp = chxj_encoding(r, *src, (apr_size_t*)len);
-      dst = chxj_exchange_hdml(r, spec, tmp, *len, len, entryp);
-      break;
-
-    case CHXJ_SPEC_Jhtml:
-      /*----------------------------------------------------------------------*/
-      /* J-Phone and Vodaphone                                                */
-      /*----------------------------------------------------------------------*/
-      DBG(r,"select JHTML");
-      tmp = chxj_encoding(r, *src, (apr_size_t*)len);
-      dst = chxj_exchange_jhtml(r, spec, tmp, *len, len, entryp);
-      break;
-
-    default:
-      DBG(r, "select ?????");
-      DBG1(r,"html_spec_type[%d]", spec->html_spec_type);
-      break;
+    if (exchange_routine[spec->html_spec_type].exchange) {
+      if (tmp)
+        dst = exchange_routine[spec->html_spec_type].exchange(r, spec, tmp, *len, len, entryp);
+      else
+        dst = exchange_routine[spec->html_spec_type].exchange(r, spec, *src, *len, len, entryp);
     }
   }
 
@@ -288,50 +326,27 @@ chxj_exchange_input_header(request_rec *r,chxjconvrule_entry* entryp)
  * @param len [i/o] It is length of former HTML character string.
  */
 static char*
-chxj_input_exchange(request_rec *r, const char** src, apr_size_t* len)
+chxj_input_exchange(
+  request_rec *r, 
+  const char** src, 
+  apr_size_t* len, 
+  chxjconvrule_entry* entryp)
 {
   char* pair;
   char* name;
   char* value;
   char* pstate;
   char* vstate;
-  char* user_agent;
-  device_table* spec ;
   char* s = apr_pstrdup(r->pool, *src);
 
   char* result;
 
-  mod_chxj_config* dconf;
-  chxjconvrule_entry* entryp;
-
-  dconf = ap_get_module_config(r->per_dir_config, &chxj_module);
-
-  entryp = chxj_apply_convrule(r, dconf->convrules);
-  if (!(entryp->action & CONVRULE_ENGINE_ON_BIT)) {
-    DBG(r,"EngineOff");
-    return (char*)*src;
-  }
-  user_agent = (char*)apr_table_get(r->headers_in, HTTP_USER_AGENT);
-  spec = chxj_specified_device(r, user_agent);
-
-  switch(spec->html_spec_type) {
-  case CHXJ_SPEC_Chtml_1_0:
-  case CHXJ_SPEC_Chtml_2_0:
-  case CHXJ_SPEC_Chtml_3_0:
-  case CHXJ_SPEC_Chtml_4_0:
-  case CHXJ_SPEC_Chtml_5_0:
-  case CHXJ_SPEC_XHtml_Mobile_1_0:
-  case CHXJ_SPEC_Hdml:
-  case CHXJ_SPEC_Jhtml:
-    break;
-  default:
-    return (char*)*src;
-  }
-
   chxj_exchange_input_header(r, entryp);
 
   result = qs_alloc_zero_byte_string(r);
 
+  DBG1(r, "BEFORE input exchange source = [%s]", s);
+
   /* _chxj_dmy */
   /* _chxj_c_ */
   /* _chxj_r_ */
@@ -381,10 +396,31 @@ chxj_input_exchange(request_rec *r, const char** src, apr_size_t* len)
       if (strlen(result) != 0)
         result = apr_pstrcat(r->pool, result, "&", NULL);
 
-      result = apr_pstrcat(r->pool, result, &name[8], "=", value, NULL);
+      if (strcasecmp(entryp->encoding, "NONE") != 0 && value && strlen(value)) {
+        apr_size_t dlen;
+        char* dvalue;
+
+        dlen   = strlen(value);
+        ap_unescape_url(value);
+        DBG1(r, "************ before encoding[%s]", value);
+
+        dvalue = chxj_rencoding(r, value, &dlen);
+        dvalue = ap_escape_uri(r->pool,dvalue);
+
+        DBG1(r, "************ after encoding[%s]", dvalue);
+
+        result = apr_pstrcat(r->pool, result, &name[8], "=", dvalue, NULL);
+
+      }
+      else {
+        result = apr_pstrcat(r->pool, result, &name[8], "=", value, NULL);
+      }
     }
   }
   *len = strlen(result);
+
+  DBG1(r, "AFTER input exchange result = [%s]", result);
+
   return result;
 }
 
@@ -461,10 +497,15 @@ chxj_output_filter(ap_filter_t *f, apr_bucket_brigade *bb)
       /* End Of File                                                          */
       /*----------------------------------------------------------------------*/
       if (f->ctx) {
+        DBG(r, " ");
         ctx = (mod_chxj_ctx*)f->ctx;
-        if (*(char*)r->content_type == 't' 
+        DBG1(r, "content_type=[%s]", r->content_type);
+        if (r->content_type 
+        && *(char*)r->content_type == 't' 
         && strncmp(r->content_type, "text/html",   9) == 0) {
+          DBG(r, " ");
           if (ctx->len) {
+            DBG(r, " ");
             char* tmp = apr_palloc(r->pool, ctx->len + 1);
             memset(tmp, 0, ctx->len + 1);
             memcpy(tmp, ctx->buffer, ctx->len);
@@ -476,13 +517,19 @@ chxj_output_filter(ap_filter_t *f, apr_bucket_brigade *bb)
             DBG2(r, "output data=[%.*s]", ctx->len,ctx->buffer);
           }
           else {
+            DBG(r, " ");
             ctx->buffer = apr_psprintf(r->pool, "\n");
             ctx->len += 1;
+            DBG(r, " ");
           }
         }
-        if (*(char*)r->content_type == 'i' && 
-        strncmp(r->content_type, "image/", 6) == 0) {
+        DBG(r, " ");
+        if (r->content_type 
+        && *(char*)r->content_type == 'i' 
+        && strncmp(r->content_type, "image/", 6) == 0) {
+          DBG(r, " ");
           if (ctx->len) {
+            DBG(r, " ");
             char* tmp = apr_palloc(r->pool, ctx->len + 1);
             memset(tmp, 0, ctx->len + 1);
             memcpy(tmp, ctx->buffer, ctx->len);
@@ -497,19 +544,25 @@ chxj_output_filter(ap_filter_t *f, apr_bucket_brigade *bb)
 
             DBG2(r, "output data=[%.*s]", ctx->len,ctx->buffer);
           }
+          DBG(r, " ");
         }
+        DBG(r, " ");
         contentLength = apr_psprintf(r->pool, "%d", ctx->len);
         apr_table_setn(r->headers_out, "Content-Length", contentLength);
+        DBG(r, " ");
         
         if (ctx->len > 0) {
           rv = pass_data_to_filter(f, (const char*)ctx->buffer, (apr_size_t)ctx->len);
         }
         f->ctx = NULL;
+        DBG(r, " ");
         return rv;
       }
       else {
+        DBG(r, " ");
         apr_table_setn(r->headers_out, "Content-Length", "0");
         rv = pass_data_to_filter(f, (const char*)"", (apr_size_t)0);
+        DBG(r, " ");
         return rv;
       }
     }
@@ -595,6 +648,11 @@ chxj_input_filter(ap_filter_t*        f,
   const char*         data;
   char*               data_bucket;
   char*               data_brigade;
+  char*               content_type;
+  device_table*       spec ;
+  char*               user_agent;
+  mod_chxj_config* dconf;
+  chxjconvrule_entry* entryp;
 
   DBG(r, "start of chxj_input_filter()");
 
@@ -604,6 +662,41 @@ chxj_input_filter(ap_filter_t*        f,
   ibb = apr_brigade_create(r->pool, c->bucket_alloc);
   obb = apr_brigade_create(r->pool, c->bucket_alloc);
 
+  content_type = (char*)apr_table_get(r->headers_in, "Content-Type");
+  if (content_type && strncasecmp("multipart/form-data", content_type, 19) == 0) {
+    DBG(r, "detect multipart/form-data");
+    ap_remove_input_filter(f);
+    return ap_get_brigade(f->next, bb, mode, block, readbytes);
+  }
+
+  dconf = ap_get_module_config(r->per_dir_config, &chxj_module);
+
+  entryp = chxj_apply_convrule(r, dconf->convrules);
+  if (!entryp || !(entryp->action & CONVRULE_ENGINE_ON_BIT)) {
+    DBG(r,"EngineOff");
+    ap_remove_input_filter(f);
+    return ap_get_brigade(f->next, bb, mode, block, readbytes);
+  }
+
+  user_agent = (char*)apr_table_get(r->headers_in, "User-Agent");
+  spec = chxj_specified_device(r, user_agent);
+
+  switch(spec->html_spec_type) {
+  case CHXJ_SPEC_Chtml_1_0:
+  case CHXJ_SPEC_Chtml_2_0:
+  case CHXJ_SPEC_Chtml_3_0:
+  case CHXJ_SPEC_Chtml_4_0:
+  case CHXJ_SPEC_Chtml_5_0:
+  case CHXJ_SPEC_XHtml_Mobile_1_0:
+  case CHXJ_SPEC_Hdml:
+  case CHXJ_SPEC_Jhtml:
+    break;
+  default:
+    ap_remove_input_filter(f);
+    return ap_get_brigade(f->next, bb, mode, block, readbytes);
+  }
+
+
   rv = ap_get_brigade(f->next, ibb, mode, block, readbytes);
   if (rv != APR_SUCCESS) {
     DBG(r, "ap_get_brigade() failed");
@@ -632,6 +725,7 @@ chxj_input_filter(ap_filter_t*        f,
   }
   apr_brigade_cleanup(ibb);
 
+
   len = strlen(data_brigade);
   if (len == 0) {
     DBG(r,"data_brigade length is 0");
@@ -639,7 +733,12 @@ chxj_input_filter(ap_filter_t*        f,
     ap_remove_input_filter(f);
     return ap_get_brigade(f->next, bb, mode, block, readbytes);
   }
-  data_brigade = chxj_input_exchange(r, (const char**)&data_brigade, (apr_size_t*)&len);
+  data_brigade = chxj_input_exchange(
+    r, 
+    (const char**)&data_brigade, 
+    (apr_size_t*)&len,
+    entryp
+    );
 
   if (len > 0) {
     DBG1(r, "(in:exchange)POSTDATA:[%s]", data_brigade);
@@ -794,6 +893,7 @@ chxj_register_hooks(apr_pool_t *p)
   ap_hook_handler(chxj_img_conv_format_handler, NULL, NULL, APR_HOOK_MIDDLE);
   ap_hook_handler(chxj_qr_code_handler, NULL, NULL, APR_HOOK_MIDDLE);
   ap_hook_translate_name(chxj_translate_name, NULL, NULL, APR_HOOK_MIDDLE);
+  ap_hook_fixups(chxj_headers_fixup, NULL, NULL, APR_HOOK_LAST);
 }
 
 /**
@@ -906,7 +1006,13 @@ chxj_merge_per_dir_config(apr_pool_t *p, void *basev, void *addv)
 
 
 static int
-chxj_command_parse_take4(const char* arg, char** prm1, char** prm2, char** prm3, char** prm4)
+chxj_command_parse_take5(
+  const char* arg, 
+  char** prm1, 
+  char** prm2, 
+  char** prm3, 
+  char** prm4, 
+  char** prm5)
 {
   int isquoted;
   char* strp;
@@ -939,6 +1045,7 @@ chxj_command_parse_take4(const char* arg, char** prm1, char** prm2, char** prm3,
     *prm2 = strp;
     *prm3 = strp;
     *prm4 = strp;
+    *prm5 = strp;
     return 1;
   }
 
@@ -968,6 +1075,7 @@ chxj_command_parse_take4(const char* arg, char** prm1, char** prm2, char** prm3,
   if (! *strp) {
     *prm3 = strp;
     *prm4 = strp;
+    *prm5 = strp;
     return 0;
   }
 
@@ -992,10 +1100,10 @@ chxj_command_parse_take4(const char* arg, char** prm1, char** prm2, char** prm3,
     ||  (isquoted  && *strp == '"'))
       break;
   }
-  *strp = '\0';
 
   if (! *strp) {
     *prm4 = strp;
+    *prm5 = strp;
     return 0;
   }
 
@@ -1020,6 +1128,33 @@ chxj_command_parse_take4(const char* arg, char** prm1, char** prm2, char** prm3,
     ||  (isquoted  && *strp == '"'))
       break;
   }
+
+  if (! *strp) {
+    *prm5 = strp;
+    return 0;
+  }
+
+  *strp++ = '\0';
+
+  for (;*strp == ' '||*strp == '\t'; strp++);
+
+  isquoted = 0; 
+  if (*strp == '"') { 
+    isquoted = 1;
+    strp++;
+  }
+  *prm5 = strp;
+  for (; *strp != '\0'; strp++) {
+    if ((isquoted && (*strp == ' ' || *strp == '\t'))
+    ||  (*strp == '\\' && (*(strp+1) == ' ' || *(strp+1) == '\t'))) {
+      strp++;
+      continue;
+    }
+
+    if ((!isquoted && (*strp == ' ' || *strp == '\t'))
+    ||  (isquoted  && *strp == '"'))
+      break;
+  }
   *strp = '\0';
 
   return 0;
@@ -1150,6 +1285,7 @@ cmd_convert_rule(cmd_parms *cmd, void* mconfig, const char *arg)
   char* prm2;
   char* prm3;
   char* prm4;
+  char* prm5;
   int mode;
   char* pstate;
   char* action;
@@ -1166,12 +1302,13 @@ cmd_convert_rule(cmd_parms *cmd, void* mconfig, const char *arg)
   if (dconf->convrules == NULL)
     dconf->convrules   = apr_array_make(cmd->pool, 2, sizeof(chxjconvrule_entry));
 
+
   newrule = apr_array_push(dconf->convrules);
 
   newrule->flags  = 0;
   newrule->action = 0;
 
-  if (chxj_command_parse_take4(arg, &prm1, &prm2, &prm3, &prm4))
+  if (chxj_command_parse_take5(arg, &prm1, &prm2, &prm3, &prm4, &prm5))
     return "ChxjConvertRule: bad argument line";
 
   newrule->pattern = apr_pstrdup(cmd->pool, prm1);
@@ -1215,7 +1352,11 @@ cmd_convert_rule(cmd_parms *cmd, void* mconfig, const char *arg)
   if (*prm4)
     if (strcasecmp(CONVRULE_PC_FLAG_ON_CMD, prm4) == 0)
       newrule->pc_flag = CONVRULE_PC_FLAG_ON_BIT;
-  
+
+  newrule->user_agent = NULL;
+  if (*prm5)
+    newrule->user_agent = apr_pstrdup(cmd->pool, prm5);
+    
   return NULL;
 }
 
