@@ -15,7 +15,6 @@
  * limitations under the License.
  */
 #include <stdio.h>
-#include "qs_malloc.h"
 #include "qs_parse_string.h"
 #include "qs_log.h"
 #include "qs_parse_attr.h"
@@ -43,7 +42,7 @@ qs_parse_tag(Doc* doc, const char* s, int len)
 
   node = (Node*)qs_new_tag(doc);
   node->name = tag_name;
-  node->otext = qs_malloc(doc,len+2, QX_LOGMARK);
+  node->otext = apr_palloc(doc->pool,len+2);
   memset(node->otext, 0, len+2);
   memcpy(node->otext, sp, len+1);
 
@@ -52,11 +51,9 @@ qs_parse_tag(Doc* doc, const char* s, int len)
   ll -= (strlen(tag_name));
   QX_LOGGER_DEBUG_INT("ll",ll);
   sp += (strlen(tag_name)+1);
-  for (;;) 
-  {
+  for (;;) {
     Attr* attr = qs_parse_attr(doc,sp, ll, &next_point);
-    if (attr == NULL) 
-    {
+    if (attr == NULL) {
       QX_LOGGER_DEBUG("End of QS_PARSE_ATTR()");
       break;
     }
@@ -84,26 +81,21 @@ s_get_tag_name(Doc* doc, const char* s, int len)
   char* return_value = NULL;
 
   /* ignore space. */
-  for (ii = 0; ii < len; ii++) 
-  {
+  for (ii = 0; ii < len; ii++) {
     if (is_white_space(s[ii])) 
-    {
       continue;
-    }
     break;
   }
+
   sp = ii;
-  for (;ii<len; ii++) 
-  {
+  for (;ii<len; ii++) {
     if (is_white_space(s[ii])) 
-    {
       break;
-    }
   }
 
   size = ii-sp;
 
-  return_value = (char*)qs_malloc(doc, size+1,QX_LOGMARK);
+  return_value = (char*)apr_palloc(doc->pool, size+1);
 
   memset(return_value, 0, size+1);
   memcpy(return_value, &s[sp], size);
@@ -117,7 +109,7 @@ s_get_tag_name(Doc* doc, const char* s, int len)
 Node*
 qs_new_tag(Doc* doc) 
 {
-  Node* node      = (Node*)qs_malloc(doc, sizeof(Node), QX_LOGMARK);
+  Node* node      = (Node*)apr_palloc(doc->pool, sizeof(Node));
   node->next      = NULL;
   node->parent    = NULL;
   node->child     = NULL;
@@ -136,15 +128,13 @@ qs_new_tag(Doc* doc)
 Node*
 qs_add_attr(Doc* doc, Node* node, Attr* attr) 
 {
-  if (node == NULL) 
-  {
+  if (node == NULL) {
     QX_LOGGER_FATAL("qs_add_attr() node is null");
   }
 
   attr->parent = node;
   attr->next   = NULL;
-  if (node->attr == NULL) 
-  {
+  if (node->attr == NULL) {
     node->attr      = attr;
     node->attr_tail = attr;
     return node;
